/**
* ML407 PAM Manager Library header.
*/
#define mlclk_api_MAJOR 2
#define mlclk_api_MINOR 2
#define mlclk_api_REVISION 1

// Types
#define	byte	unsigned char
#define	UInt16	unsigned short
#define	UInt32	unsigned long
#define UInt64  unsigned long long


struct mlclk_api_version {
	int major;
	int minor;
	int revision;
};

struct mlclk_api;

#ifndef MLCLKCALL
#ifdef __cplusplus
#	ifdef _WIN32
#		ifndef ML_IMPORTS_API
#			define MLCLKCALL extern "C" __declspec(dllexport)
#		else
#			define MLCLKCALL extern "C" __declspec(dllimport)
#		endif
#	else
#	define MLCLKCALL extern "C"
#	endif // WIN32
#else
#	define MLCLKCALL
#endif
#endif

#ifdef __cplusplus
#	ifdef _WIN32
//#define MLCLKSTACKMODE __stdcall
#		 define MLCLKSTACKMODE __cdecl
#	else
#		define MLCLKSTACKMODE
#	endif //WIN32
#else
#	define MLCLKSTACKMODE
#endif

typedef struct mlclkapi mlclkapi;

/// <summary>
/// API status enumerator
/// </summary>
typedef enum MLCLK_STATUS
{
	MLCLK_SUCCESS = 0,			// Operation successful
	MLCLK_FAILED = 1,			// Operation failed
	MLCLK_NOT_CONNECTED = 2,	// Device not connected
	MLCLK_INVALID_INPUT = 3,	// Invalid input provided
	MLCLK_INVALID_INSTANCE = 4, // mlclk_api instance = 0
	MLCLK_WRONG_USE = 5			// Returned when custom frequency is used with other than no calibration
}MLCLK_STATUS;

/// <summary>
/// API calibration type enumerator
/// </summary>
typedef enum MLCLK_CALIBRATION
{
	MLCLK_NO_CALIBRATION,
	MLCLK_FACTORY_CALIBRATION,
	MLCLK_CUSTOM_CALIBRATION
}MLCLK_CALIBRATION;

/// <summary>
/// API Bandwidth enumerator
/// </summary>
typedef enum MLCLK_BANDWIDTH
{
	MLCLK_BW_5kHz,
	MLCLK_BW_10kHz,
	MLCLK_BW_20kHz,
	MLCLK_BW_40kHz,
	MLCLK_BW_80kHz,
	MLCLK_BW_100kHz,
	MLCLK_BW_150kHz,
	MLCLK_BW_200kHz,
	MLCLK_BW_250kHz,
	MLCLK_BW_300kHz,
	MLCLK_BW_350kHz,
	MLCLK_BW_1MHz,
	MLCLK_BW_2MHz,
	MLCLK_BW_4MHz,
	MLCLK_BW_8MHz,
	MLCLK_BW_10MHz,
	MLCLK_BW_15MHz,
	MLCLK_BW_HT1,
	MLCLK_BW_HT2,
	MLCLK_BW_FMHIGH,
	MLCLK_BW_FMLOW,
}MLCLK_BANDWIDTH;

/// <summary>
/// API clock source enumerator
/// </summary>
typedef enum MLCLK_CLOCKSOURCE
{
	MLCLK_INTERNALCLKSRC,
	MLCLK_EXTERNALCLKSRC
}MLCLK_CLOCKSOURCE;

/// <summary>
/// API BUJ linerate enumerator
/// </summary>
typedef enum MLCLK_BUJLINERATE
{
	MLCLK_10Gbps,
	MLCLK_5Gbps,
	MLCLK_2p5Gbps,
	MLCLK_1p25Gbps,
	MLCLK_0p625Gbps
}MLCLK_BUJLINERATE;

/// <summary>
/// API BUJ PRBS enumerator
/// </summary>
typedef enum MLCLK_BUJPRBS
{
	MLCLK_PRBS7,
	MLCLK_PRBS15,
	MLCLK_PRBS31,
	MLCLK_USERDEFINED
}MLCLK_BUJPRBS;

/// <summary>
/// API Tx2 divider enumerator
/// </summary>
typedef enum MLCLK_TX2DIV
{
	MLCLK_TX2DIV_1,
	MLCLK_TX2DIV_2,
	MLCLK_TX2DIV_4,
	MLCLK_TX2DIV_8
}MLCLK_TX2DIV;

/// <summary>
/// API FM frequency enumerator
/// </summary>
typedef enum MLCLK_FMFREQ
{
	MLCLK_40kHz = 0,
	MLCLK_100kHz = 1,
	MLCLK_500kHz = 2,
	MLCLK_1MHz = 3,
	MLCLK_1p3MHz = 4,
	MLCLK_2MHz = 5,
	MLCLK_2p5MHz = 6,
	MLCLK_3MHz = 7,
	MLCLK_3p5MHz = 8,
	MLCLK_4MHz = 9,
	MLCLK_10MHz = 10,
	MLCLK_50MHz = 11
}MLCLK_FMFREQ;

struct ConfigurationSettings
{
	MLCLK_CLOCKSOURCE ClockSource;
	int ClockOutDivider;
	MLCLK_BANDWIDTH Bandwith;
	double OutputFrequency;
	bool FM_enable;
	int FM_Frequency;
	int FM_Amplitude;
	int FMPhaseShift_Amplitude;
	int FMRJ_Amplitude;
	bool PM_Enable;
	int PM_Frequency;
	int PM_Amplitude;
	int PMPhase_Shift;
	bool PMRJ_Enable;
	int PMRJ_Amplitude;
	bool PM_BUJ_Enable;
	int PM_BUJ_Amplitude;
	MLCLK_BUJLINERATE PM_BUJ_LineRate;
	bool AM_Enable;
	int AM_Amplitude;
	int AMRJ_Amplitude;
	int BUJ_Amplitude;
	MLCLK_BUJPRBS BUJ_PRBS;
	bool BUJ_Invert;
	double ExternalClockReference;
	MLCLK_TX2DIV TX2_Divider;
	int TX1POWER;
	int TX2POWER;
};

/// <summary>
/// Get a runtime version of the library
/// </summary>
/// <returns>API version struct</returns>
MLCLKCALL mlclk_api_version MLCLKSTACKMODE mlclk_APIVersion();

/// <summary>
/// Create a new ML-407 API instance
/// </summary>
/// <returns>
/// Pointer to created instance
/// </returns>
MLCLKCALL mlclk_api* MLCLKSTACKMODE mlclk_createInstance();

/// <summary>
/// Destroy the ML-407 API instance
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
MLCLKCALL void MLCLKSTACKMODE mlclk_destroyInstance(mlclk_api* instance);

/// <summary>
/// Open and initializes connection to the hardware
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="IPAddress">The address to connect to</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_connect(mlclk_api* instance, char IPAddress[]);

/// <summary>
/// Disconnect instance
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_disconnect(mlclk_api* instance);

/// <summary>
/// chooses calibration type
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="calibration">calibration type</param>
/// <param name="customPath">path to custom calibration file</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_initializeInstance(mlclk_api* instance, MLCLK_CALIBRATION calibration, char customPath[]);

/// <summary>
/// Read Board ID
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="boardID">Pointer to ML-407 API boardID</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_getBoardID(mlclk_api* instance, int *boardID);

/// <summary>
/// Read Serial Number
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="SN">Pointer to ML-407 API serial number</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_getSerialNumber(mlclk_api* instance, UInt64 *SN);

/// <summary>
/// Read Temperature
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="index">Temperature index(1 or 0)</param>
/// <param name="temperature">Pointer to ML-407 API temperature</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_getTemperature(mlclk_api* instance, int index, int* temperature);

/// <summary>
/// Reference Clock
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="clockSource">Clock source type(MLCLK_INTERNALCLKSRC or MLCLK_EXTERNALCLKSRC)</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setClockSource(mlclk_api* instance, MLCLK_CLOCKSOURCE clockSource);

/// <summary>
/// Configure and generate clock output
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="clkvalue">Clock value</param>
/// <param name="bandwidth">Bandwidth type</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_generateClock(mlclk_api* instance, double clkvalue, MLCLK_BANDWIDTH bandwidth);

/// <summary>
/// Check VCO lock
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="VCOLockStatus">Pointer to ML-407 API VCO Lock status</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_checkVCOLock(mlclk_api* instance, int* VCOLockStatus);

/// <summary>
/// Check VCO Voltage
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="VCOVoltage">Pointer to ML-407 API VCO Voltage</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_checkVCOVoltage(mlclk_api* instance, double* VCOVoltage);

/// <summary>
/// Enable/Disable FM
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="FM_Status">FM Status enabled =0 , disabled = 1</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_enableFM(mlclk_api* instance, int FM_Status);

/// <summary>
/// Set FM Frequency
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="FM_Frequency">FM Frequency value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setFMFrequency(mlclk_api* instance, MLCLK_FMFREQ FM_Frequency);

/// <summary>
/// Set FM Frequency
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="FM_Frequency">FM Frequency value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setCustomFMFrequency(mlclk_api* instance, double FM_Frequency);

/// <summary>
/// Set FM Amplitude
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="FMamplitude">FM amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setFMAmplitude(mlclk_api* instance, int FMamplitude);

/// <summary>
/// Set FM Phase Shift Amplitude
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="fmshiftamplitude">FM shift amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setFMPhaseSkew(mlclk_api* instance, int fmshiftamplitude);

/// <summary>
/// Set FM RJ Amplitude
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="FMRJ_Amplitude_">FM RJ Amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setFMRJAmplitude(mlclk_api* instance, int FMRJ_Amplitude_);

/// <summary>
/// Tx1 power
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="tx1pow">Tx1 power value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx1Power(mlclk_api* instance, int tx1pow);

/// <summary>
/// Tx2 power
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="tx2pow">Tx2 power value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2Power(mlclk_api* instance, int tx2pow);

/// <summary>
/// Tx2 divider
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="Tx2Divider">Tx2 divider value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2Divider(mlclk_api* instance, int Tx2Divider);

/// <summary>
/// Gets current configurations from device
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="mlclkstruct">Pointer to ML-407 API mlclkstruct</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_getActiveConfig(mlclk_api* instance, ConfigurationSettings *config);

/// <summary>
/// Gets current configurations from device
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="AMStatus">AM Status enabled =0 , disabled = 1</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_enableTx2AM(mlclk_api* instance, int AMStatus);


/// <summary>
/// Gets current configurations from device
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="amamplitude">AM amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2AMAmplitude(mlclk_api* instance, int amamplitude);

/// <summary>
/// Gets current configurations from device
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="AMRJAmplitude">AM RJ Amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2AMRJAmplitude(mlclk_api* instance, int AMRJAmplitude);

/// <summary>
/// Enable/Disable PM
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="PMStatus">PM Status enabled =0 , disabled = 1</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_enableTx2PM(mlclk_api* instance, int PMStatus);

/// <summary>
/// Set PM Frequency
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="PM_Frequency">PM frequency value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2PMFrequency(mlclk_api* instance, double PMFrequency);

/// <summary>
/// Set PM Amplitude
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="pmamplitude">PM amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2PMAmplitude(mlclk_api* instance, int PMAmplitude);

/// <summary>
/// Set PM Phase Shift Amplitude
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="pmshiftamplitude">PM phase shift amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2PMPhaseShiftAmplitude(mlclk_api* instance, int PMShiftAmplitude);

/// <summary>
/// Enable/Disable PM RJ
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="PMRJStatus">PM RJ Status enabled =0 , disabled = 1</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_enableTx2PMRJ(mlclk_api* instance, int PMRJStatus);

/// <summary>
/// Set PM RJ Amplitude
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="PMRJ_Amplitude">PM RJ amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2PMRJAmplitude(mlclk_api* instance, int PMRJAmplitude);

/// <summary>
/// Enable/Disable PM BUJ
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="PM_BUJ_Status">PM BUJ Status enabled =0 , disabled = 1</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_enableTx2PMBUJ(mlclk_api* instance, int PMBUJStatus);

/// <summary>
/// Set PM BUJ Amplitude
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="PM_BUJ_Amplitude">PM BUJ amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2PMBUJAmplitude(mlclk_api* instance, int PMBUJAmplitude);

/// <summary>
/// Set PM BUJ LineRateGeneration
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="LineRateIndex">PM BUJ LineRateIndex value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2PMBUJLineRate(mlclk_api* instance, int LineRateIndex);

/// <summary>
/// Set PM BUJ Amplitude
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="BUJ_Amplitude">BUJ amplitude value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2BUJAmplitude(mlclk_api* instance, int BUJAmplitude);

/// <summary>
/// Set BUJ PRBS
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="BUJ_PRBS_Index">BUJ PRBS index</param>
/// <param name="Invert">invert BUJ PRBS value</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2BUJPRBS(mlclk_api* instance, int BUJPRBSIndex, int Invert);

/// <summary>
/// Set BUJ UserDefinedPattern
/// </summary>
/// <param name="instance">Pointer to ML-407 API instance</param>
/// <param name="BUJ_PRBS_Index">BUJ PRBS index for user defined pattern</param>
/// <returns>Operation status</returns>
MLCLKCALL MLCLK_STATUS MLCLKSTACKMODE mlclk_setTx2BUJUserDefinedPattern(mlclk_api* instance, UInt32 BUJPRBSIndex);
